Package: textreuse
Type: Package
Title: Detect Text Reuse and Document Similarity
Version: 1.0.2
Date: 2026-07-24
Authors@R: c(
    person("Lincoln", "Mullen", role = "aut",
        comment = c(ORCID = "0000-0001-5103-6917")
    ),
    person("Yaoxiang", "Li", role = c("aut", "cre"),
        email = "liyaoxiang@outlook.com",
        comment = c(ORCID = "0000-0001-9200-1016")))
Description: Tools for measuring similarity among documents and detecting
    passages which have been reused. Implements shingled n-gram, skip n-gram,
    and other tokenizers; similarity/dissimilarity functions; pairwise
    comparisons; minhash and locality sensitive hashing algorithms; and a
    version of the Smith-Waterman local alignment algorithm suitable for
    natural language.
License: MIT + file LICENSE
URL: https://docs.ropensci.org/textreuse/,
        https://github.com/ropensci/textreuse
BugReports: https://github.com/ropensci/textreuse/issues
VignetteBuilder: knitr
Depends: R (>= 3.1.1)
Imports: assertthat (>= 0.1), digest (>= 0.6.8), dplyr (>= 1.1.0), NLP
        (>= 0.1.8), Matrix, Rcpp (>= 0.12.0), RcppProgress (>= 0.1),
        stringr (>= 1.0.0), tibble (>= 3.0.1), tidyr (>= 1.0.0)
Suggests: testthat (>= 0.11.0), knitr (>= 1.11), rmarkdown (>= 0.8),
        covr
LinkingTo: BH, Rcpp, RcppProgress
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-07-24 22:50:27 UTC; Li
Author: Lincoln Mullen [aut] (ORCID: <https://orcid.org/0000-0001-5103-6917>),
  Yaoxiang Li [aut, cre] (ORCID: <https://orcid.org/0000-0001-9200-1016>)
Maintainer: Yaoxiang Li <liyaoxiang@outlook.com>
Repository: CRAN
Date/Publication: 2026-07-25 17:10:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-07-25 19:53:43 UTC; unix
Archs: textreuse.so.dSYM
