# Generated by roxygen2: do not edit by hand

S3method("[",TextReuseCorpus)
S3method("[[",TextReuseCorpus)
S3method("content<-",TextReuseTextDocument)
S3method("hashes<-",TextReuseTextDocument)
S3method("meta<-",TextReuseCorpus)
S3method("meta<-",TextReuseTextDocument)
S3method("minhashes<-",TextReuseTextDocument)
S3method("names<-",TextReuseCorpus)
S3method("tokens<-",TextReuseTextDocument)
S3method(align_local,TextReuseTextDocument)
S3method(align_local,default)
S3method(as.character,TextReuseTextDocument)
S3method(as.matrix,textreuse_candidates)
S3method(content,TextReuseTextDocument)
S3method(count_matches,TextReuseTextDocument)
S3method(count_matches,default)
S3method(hashes,TextReuseCorpus)
S3method(hashes,TextReuseTextDocument)
S3method(jaccard_bag_similarity,TextReuseTextDocument)
S3method(jaccard_bag_similarity,default)
S3method(jaccard_dissimilarity,default)
S3method(jaccard_similarity,TextReuseTextDocument)
S3method(jaccard_similarity,default)
S3method(length,TextReuseCorpus)
S3method(lsh,TextReuseCorpus)
S3method(lsh,TextReuseTextDocument)
S3method(matching_tokens,TextReuseTextDocument)
S3method(matching_tokens,default)
S3method(meta,TextReuseCorpus)
S3method(meta,TextReuseTextDocument)
S3method(minhashes,TextReuseCorpus)
S3method(minhashes,TextReuseTextDocument)
S3method(names,TextReuseCorpus)
S3method(print,TextReuseCorpus)
S3method(print,TextReuseTextDocument)
S3method(print,textreuse_alignment)
S3method(ratio_of_matches,TextReuseTextDocument)
S3method(ratio_of_matches,default)
S3method(rehash,TextReuseCorpus)
S3method(rehash,TextReuseTextDocument)
S3method(tokenize,TextReuseCorpus)
S3method(tokenize,TextReuseTextDocument)
S3method(tokens,TextReuseCorpus)
S3method(tokens,TextReuseTextDocument)
S3method(wordcount,TextDocument)
S3method(wordcount,TextReuseCorpus)
S3method(wordcount,default)
export("content<-")
export("hashes<-")
export("meta<-")
export("minhashes<-")
export("tokens<-")
export(TextReuseCorpus)
export(TextReuseTextDocument)
export(align_local)
export(as_sparse_matrix)
export(content)
export(count_matches)
export(filenames)
export(has_content)
export(has_hashes)
export(has_minhashes)
export(has_tokens)
export(hash_string)
export(hashes)
export(is.TextReuseCorpus)
export(is.TextReuseTextDocument)
export(jaccard_bag_similarity)
export(jaccard_dissimilarity)
export(jaccard_similarity)
export(lsh)
export(lsh_add)
export(lsh_candidates)
export(lsh_compare)
export(lsh_probability)
export(lsh_query)
export(lsh_subset)
export(lsh_threshold)
export(matching_tokens)
export(meta)
export(minhash_generator)
export(minhashes)
export(pairwise_candidates)
export(pairwise_compare)
export(ratio_of_matches)
export(rehash)
export(skipped)
export(token_index)
export(token_index_candidates)
export(tokenize)
export(tokenize_ngrams)
export(tokenize_sentences)
export(tokenize_skip_ngrams)
export(tokenize_words)
export(tokens)
export(wordcount)
import(RcppProgress)
import(assertthat)
import(stringr)
importFrom(NLP,"content<-")
importFrom(NLP,"meta<-")
importFrom(NLP,content)
importFrom(NLP,meta)
importFrom(Rcpp,sourceCpp)
importFrom(utils,getTxtProgressBar)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(textreuse, .registration = TRUE)
